/*
 * Copyright 2011 Objectos, Fábrica de Software LTDA.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package br.com.objectos.blog.syntax;

import static com.google.common.collect.Lists.transform;

import java.io.File;
import java.io.IOException;
import java.util.List;

import com.google.common.base.Charsets;
import com.google.common.base.Function;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableList.Builder;
import com.google.common.io.Files;

/**
 * @author marcio.endo@objectos.com.br (Marcio Endo)
 */
public class SyntaxAnalyzer {

  private static final List<SyntaxFactory> syntaxes;

  static {
    Builder<SyntaxFactory> builder = ImmutableList.<SyntaxFactory> builder();

    builder.add(Meta.factory());
    builder.add(BlockquoteStart.factory());
    builder.add(BlockquoteEnd.factory());
    builder.add(Heading.factory());
    builder.add(Separator.factory());
    builder.add(Paragraph.factory());
    builder.add(ParagraphBreak.factory());

    syntaxes = builder.build();
  }

  private final File file;

  public SyntaxAnalyzer(File file) {
    this.file = file;
  }

  public List<Syntax> analyze() throws SyntaxException {
    List<String> lines = readLines();

    List<Syntax> result = transform(lines, new Analyzer());

    return result;
  }

  private List<String> readLines() throws SyntaxException {
    try {

      return Files.readLines(file, Charsets.UTF_8);

    } catch (IOException e) {

      throw new SyntaxException(e);

    }
  }

  private class Analyzer implements Function<String, Syntax> {

    @Override
    public Syntax apply(String text) {
      Syntax syntax = null;

      for (SyntaxFactory factory : syntaxes) {
        syntax = factory.get(text);

        if (syntax.matches()) {
          break;
        }
      }

      return syntax;
    }

  }

}