/*
 * Copyright 2011 Objectos, Fábrica de Software LTDA.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package br.com.objectos.blog;

import static org.testng.Assert.assertEquals;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;

import org.joda.time.DateTime;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

/**
 * @author marcio.endo@objectos.com.br (Marcio Endo)
 */
@Test
public class PostTemplateLoaderTest {

  private PostTemplateLoader loader;

  @BeforeClass
  public void prepareTemplate() {
    loader = new PostTemplateLoaderGuice();
  }

  public void postIdMustHaveBeenSet() throws IOException {
    String html = "<body><script>{{post_id}}</script></body>";
    PostTemplate template = load(html);

    String postId = "000002";
    template.setPostId(postId);

    BufferedReader r = toReader(template);
    assertLine(r, "<html>");
    assertLine(r, "<head></head>");
    assertLine(r, "<body>");
    assertLine(r, "<script>");
    assertLine(r, "000002");
    assertLine(r, "</script>");
    assertLine(r, "</body>");
    assertLine(r, "</html>");
  }

  public void titleMustHaveBeenSet() throws IOException {
    String html = "<h1>{{title}}</h1>";
    PostTemplate template = load(html);

    String title = "My title";
    template.setTitle(title);

    BufferedReader r = toReader(template);
    assertLine(r, "<html>");
    assertLine(r, "<head></head>");
    assertLine(r, "<body>");
    assertLine(r, "<h1>My title</h1>");
    assertLine(r, "</body>");
    assertLine(r, "</html>");
  }

  public void weShouldBeAbleToUseTheTitleTagTwice() throws IOException {
    String html = "<title>{{title}}</title><h1>{{title}}</h1>";
    PostTemplate template = load(html);

    String title = "twice!";
    template.setTitle(title);

    BufferedReader r = toReader(template);
    assertLine(r, "<html>");
    assertLine(r, "<head>");
    assertLine(r, "<title>twice!</title>");
    assertLine(r, "</head>");
    assertLine(r, "<body>");
    assertLine(r, "<h1>twice!</h1>");
    assertLine(r, "</body>");
    assertLine(r, "</html>");
  }

  public void inFactWeShouldBeAbleToUseATagAsManyTimesAsRequired()
      throws IOException {

    String html = "<h1>{{title}}</h1><h2>{{title}}</h2><h3>{{title}}</h3>";
    PostTemplate template = load(html);

    String title = "meh";
    template.setTitle(title);

    BufferedReader r = toReader(template);
    assertLine(r, "<html>");
    assertLine(r, "<head></head>");
    assertLine(r, "<body>");
    assertLine(r, "<h1>meh</h1>");
    assertLine(r, "<h2>meh</h2>");
    assertLine(r, "<h3>meh</h3>");
    assertLine(r, "</body>");
    assertLine(r, "</html>");

  }

  public void authorShouldHaveBeenSet() throws IOException {
    String html = "<h1>{{author}}</h1>";
    PostTemplate template = load(html);

    String author = "Mr. Moe";
    template.setAuthor(author);

    BufferedReader r = toReader(template);
    assertLine(r, "<html>");
    assertLine(r, "<head></head>");
    assertLine(r, "<body>");
    assertLine(r, "<h1>Mr. Moe</h1>");
    assertLine(r, "</body>");
    assertLine(r, "</html>");
  }

  public void postDateShouldHaveBeenSet() throws IOException {
    String html = "<h1>{{date}}</h1>";
    PostTemplate template = load(html);

    DateTime date = new DateTime("2011-06-19T18:22:24.000-03:00");
    template.setDate(date);

    BufferedReader r = toReader(template);
    assertLine(r, "<html>");
    assertLine(r, "<head></head>");
    assertLine(r, "<body>");
    assertLine(r, "<h1>19/06/2011 18:22</h1>");
    assertLine(r, "</body>");
    assertLine(r, "</html>");
  }

  public void contentsShouldHaveBeenSet() throws IOException {
    String html = "<div>{{contents}}</div>";
    PostTemplate template = load(html);

    String contents = "Cool post";
    template.setContents(contents);

    BufferedReader r = toReader(template);
    assertLine(r, "<html>");
    assertLine(r, "<head></head>");
    assertLine(r, "<body>");
    assertLine(r, "<div>");
    assertLine(r, "Cool post");
    assertLine(r, "</div>");
    assertLine(r, "</body>");
    assertLine(r, "</html>");
  }

  private void assertLine(BufferedReader r, String expected) throws IOException {
    String line = r.readLine();
    assertEquals(line.trim(), expected);
  }

  private PostTemplate load(String html) {
    return loader.load(html);
  }

  private BufferedReader toReader(PostTemplate template) {
    String tmpl = template.toString();
    StringReader stringReader = new StringReader(tmpl);
    return new BufferedReader(stringReader);
  }

}